@@ -16,9 +16,7 @@ from fons.event import Event
 from fons.reg import create_name
 import fons.log as _log
 
-import functools
 import asyncio
-import copy as _copy
 
 logger,logger2,tlogger,tloggers,tlogger0 = _log.get_standard_5(__name__)
 
@@ -95,8 +93,12 @@ class WSClient(metaclass=WSMeta):
     #        "is_private": False,
     # }
     channels = {}
-
+    
     max_total_subscriptions = None
+    max_total_connections = None
+    max_subscriptions_per_connection = None
+    # the interval between pushing (sending) individual subscriptions to the server
+    subscription_push_rate_limit = None
     
     connection_defaults = {
         # change hub_name and hub_methods only if signalr is set to True
@@ -126,15 +128,19 @@ class WSClient(metaclass=WSMeta):
         # current_time > last_message_received_timestamp + ping_after
         'ping_after': None, #in seconds
         # instead sending as raw bytes (via .ping of connection's socket)
-        # sends the ping as an ordinary message (json decodes and sends it)
-        'ping_as_message': False,
+        # sends the ping as an ordinary message
+        'ping_as_message': False, # if 'dump' then the message is json encoded first
         # Only applies if 'ping_as_message' == False, force crashes-reconnects
         # the socket if no pong frame is received within that time
         'ping_timeout': 5,
+        # accepts received message as first arg, returns either
+        # a) `None` if message was not ping, b) the pong message to be sent back
+        'pong': None,
+        'pong_as_message': True, # if 'dump' then the message is json encoded first
         'poll_interval': 0.05,
-        'max_subscriptions': None,
-        # the interval between pushing (sending) individual subscriptions to the server
-        'subscription_push_rate_limit': None,
+        # True, False, 'force' (logs and skips the object if it is not unpackable)
+        # ignored if `signalr` or `socketio` is enabled
+        'unpack_json': True, 
     }
     connection_profiles = {}
     
@@ -236,8 +242,13 @@ class WSClient(metaclass=WSMeta):
     SubscriptionHandler_cls = SubscriptionHandler
     Transport_cls = Transport
     
+    _verbose = 0 # 1, 2, 3
+    
     
     def __init__(self, config={}):
+        config = config.copy()
+        subscriptions = config.pop('subscriptions', None)  
+        
         for key in config:
             setattr(self, key, deep_update(getattr(self, key, None), config[key], copy=True))
         
@@ -258,9 +269,9 @@ class WSClient(metaclass=WSMeta):
         
         self.reload_urls()
         
-        if getattr(self,'subscriptions',None) is not None:
-            for params in self.subscriptions:
-                self.sh.add_subscription(params)
+        if subscriptions is not None:
+            for params in subscriptions:
+                self.subscribe_to(params)
         
         #self.start.__doc__ = self.tp.start.__doc__    
         #self.send.__doc__ = self.tp.send.__doc__
@@ -492,6 +503,31 @@ class WSClient(metaclass=WSMeta):
     def unsubscribe_to(self, x):
         return self.sh.remove_subscription(x)
     
+    
+    def _log(self, msg, level='DEBUG', logger=logger, add_name=True):
+        if not isinstance(msg, Exception):
+            if add_name:
+                msg = '{} - {}'.format(self.name, msg)
+            logger.log(_log.level_to_int(level), msg)
+        else:
+            logger.exception(msg)
+    
+    
+    def log(self, msg, level='DEBUG', logger=logger, add_name=True):
+        if self.verbose and self.verbose >= 1:
+            self._log(msg, level, logger, add_name)
+    
+    
+    def log2(self, msg, level='DEBUG', logger=logger, add_name=True):
+        if self.verbose and self.verbose >= 2:
+            self._log(msg, level, logger, add_name)
+    
+    
+    def log3(self, msg, level='DEBUG', logger=logger, add_name=True):
+        if self.verbose and self.verbose >= 3:
+            self._log(msg, level, logger, add_name)
+    
+    
     @property
     def subscriptions(self):
         return self.sh.subscriptions[:]
@@ -543,3 +579,12 @@ class WSClient(metaclass=WSMeta):
     @property
     def station(self):
         return self.tp.station
+    @property
+    def verbose(self):
+        return self._verbose
+    @verbose.setter
+    def verbose(self, value):
+        self._verbose = value
+        if getattr(self, 'cm', None) is not None:
+            for cnx in self.cm.connections.values():
+                cnx.verbose = value
